indexing
   description: "Concrete Subject for storing and maintaining the time of day.%
   % It notifies its observers every second."

class CLOCK_TIMER
  inherit SUBJECT
		  rename make as make_from_subject end

create make

---------------------------
feature -- Creation

make (initial_time : INTEGER) is
  -- Creation and initialization with a time offset (in seconds)
  require
    positive_initial_time: initial_time >= 0
  do
    make_from_subject
    time := initial_time
  ensure 
    time_initialized: initial_time\\86400 = 3600*hour + 60*minute + second
  end

---------------------------
feature {ANY} -- Queries

hour : INTEGER is
  -- the current hour
  do
    Result := (time//3600)\\24
  end

minute : INTEGER is
  -- the current minute
  do
    Result := (time//60)\\60
  end

second : INTEGER is
  -- the current second
  do
    Result := time\\60
  end -- second
		
---------------------------
feature {ANY} -- Commands
	
tick is
  -- Called by the internal timer to provide an accurate time base
  -- Call `notify' to inform observers of the change
  do
    time := time + tick_interval
    notify
  end

---------------------------
feature {NONE} -- Private

time : INTEGER                -- Elapsed time since initialization, in seconds
tick_interval : INTEGER is 5  -- Number of seconds between ticks

invariant
  correct_time: time\\86400 = 3600*hour + 60*minute + second

end
