indexing
   description: "Object observed by any number of Observers"

class SUBJECT

create make

---------------------------
feature {NONE} -- Creation

make is
  do
    create observers.make
  end

---------------------------
feature -- Queries

is_attached_to (o : OBSERVER) : BOOLEAN is
  -- Whether o belongs to the observers
  require
    o_not_void: o /= Void
  do
    Result := observers.has(o)
  end

---------------------------
feature -- Commands

attach (o : OBSERVER) is
  -- add `o' to observers
  require 
    o_not_void: o /= Void
    not_yet_attached: not is_attached_to(o)
  do
    observers.extend(o)
  ensure
    attached: is_attached_to(o)
  end

detach (o : OBSERVER) is
  -- remove `o' from observers
  require
    o_not_void: o /= Void
  do
    from observers.start
    until observers.after or observers.item = o
    loop
      observers.forth
    end
    observers.remove
  ensure
    not_attached: not is_attached_to(o)
  end

notify is
  -- Send an update message to each observer
  do
    from observers.start
    until observers.after
    loop
    	observers.item.update
    	observers.forth
    end
  end

---------------------------
feature {NONE} -- Private

observers : LINKED_LIST[OBSERVER]

invariant
	observers_initialized: observers /= Void

end
